import * as _lukas_j_han_sd_jwt_type from '@lukas.j.han/sd-jwt-type';
import { Base64urlString, Signer, Verifier, kbHeader, kbPayload, SDJWTCompact, Hasher, DisclosureFrame, HasherAndAlg, SaltGenerator, SDJWTConfig, KBOptions } from '@lukas.j.han/sd-jwt-type';
import { Disclosure } from '@lukas.j.han/sd-jwt-util';

type JwtData<Header extends Record<string, any>, Payload extends Record<string, any>> = {
    header?: Header;
    payload?: Payload;
    signature?: Base64urlString;
};
declare class Jwt<Header extends Record<string, any> = Record<string, any>, Payload extends Record<string, any> = Record<string, any>> {
    header?: Header;
    payload?: Payload;
    signature?: Base64urlString;
    constructor(data?: JwtData<Header, Payload>);
    static decodeJWT<Header extends Record<string, any> = Record<string, any>, Payload extends Record<string, any> = Record<string, any>>(jwt: string): {
        header: Header;
        payload: Payload;
        signature: Base64urlString;
    };
    static fromEncode<Header extends Record<string, any> = Record<string, any>, Payload extends Record<string, any> = Record<string, any>>(encodedJwt: string): Jwt<Header, Payload>;
    setHeader(header: Header): Jwt<Header, Payload>;
    setPayload(payload: Payload): Jwt<Header, Payload>;
    sign(signer: Signer): Promise<string>;
    encodeJwt(): string;
    verify(verifier: Verifier): Promise<{
        payload: Payload;
        header: Header;
    }>;
}

declare class KBJwt<Header extends kbHeader = kbHeader, Payload extends kbPayload = kbPayload> extends Jwt<Header, Payload> {
    verify(verifier: Verifier): Promise<{
        payload: Payload;
        header: Header;
    }>;
    static fromKBEncode<Header extends kbHeader = kbHeader, Payload extends kbPayload = kbPayload>(encodedJwt: string): KBJwt<Header, Payload>;
}

type SDJwtData<Header extends Record<string, any>, Payload extends Record<string, any>, KBHeader extends kbHeader = kbHeader, KBPayload extends kbPayload = kbPayload> = {
    jwt?: Jwt<Header, Payload>;
    disclosures?: Array<Disclosure<any>>;
    kbJwt?: KBJwt<KBHeader, KBPayload>;
};
declare class SDJwt<Header extends Record<string, any> = Record<string, any>, Payload extends Record<string, any> = Record<string, any>, KBHeader extends kbHeader = kbHeader, KBPayload extends kbPayload = kbPayload> {
    jwt?: Jwt<Header, Payload>;
    disclosures?: Array<Disclosure<any>>;
    kbJwt?: KBJwt<KBHeader, KBPayload>;
    constructor(data?: SDJwtData<Header, Payload, KBHeader, KBPayload>);
    static decodeSDJwt<Header extends Record<string, any> = Record<string, any>, Payload extends Record<string, any> = Record<string, any>, KBHeader extends kbHeader = kbHeader, KBPayload extends kbPayload = kbPayload>(sdjwt: SDJWTCompact, hasher: Hasher): Promise<{
        jwt: Jwt<Header, Payload>;
        disclosures: Array<Disclosure<any>>;
        kbJwt?: KBJwt<KBHeader, KBPayload>;
    }>;
    static fromEncode<Header extends Record<string, any> = Record<string, any>, Payload extends Record<string, any> = Record<string, any>, KBHeader extends kbHeader = kbHeader, KBPayload extends kbPayload = kbPayload>(encodedSdJwt: SDJWTCompact, hasher: Hasher): Promise<SDJwt<Header, Payload>>;
    present(keys: string[], hasher: Hasher): Promise<SDJWTCompact>;
    encodeSDJwt(): SDJWTCompact;
    keys(hasher: Hasher): Promise<string[]>;
    presentableKeys(hasher: Hasher): Promise<string[]>;
    getClaims<T>(hasher: Hasher): Promise<T>;
}
declare const listKeys: (obj: any, prefix?: string) => string[];
declare const pack: <T extends object>(claims: T, disclosureFrame: DisclosureFrame<T> | undefined, hash: HasherAndAlg, saltGenerator: SaltGenerator) => Promise<{
    packedClaims: any;
    disclosures: Array<Disclosure<any>>;
}>;

declare const createDecoy: (hash: HasherAndAlg, saltGenerator: SaltGenerator) => Promise<string>;

declare class SDJwtInstance {
    static DEFAULT_hashAlg: string;
    private userConfig;
    constructor(userConfig?: SDJWTConfig);
    private createKBJwt;
    private SignJwt;
    private VerifyJwt;
    issue<Payload extends object>(payload: Payload, disclosureFrame?: DisclosureFrame<Payload>, options?: {
        header?: object;
    }): Promise<SDJWTCompact>;
    present(encodedSDJwt: string, presentationKeys?: string[], options?: {
        kb?: KBOptions;
    }): Promise<SDJWTCompact>;
    verify(encodedSDJwt: string, requiredClaimKeys?: string[], requireKeyBindings?: boolean): Promise<{
        payload: unknown;
        header: Record<string, any>;
        kb?: undefined;
    } | {
        payload: unknown;
        header: Record<string, any>;
        kb: {
            payload: _lukas_j_han_sd_jwt_type.kbPayload;
            header: _lukas_j_han_sd_jwt_type.kbHeader;
        };
    }>;
    validate(encodedSDJwt: string): Promise<{
        payload: unknown;
        header: Record<string, any>;
    }>;
    config(newConfig: SDJWTConfig): void;
    encode(sdJwt: SDJwt): SDJWTCompact;
    decode(endcodedSDJwt: SDJWTCompact): Promise<SDJwt<Record<string, any>, Record<string, any>, _lukas_j_han_sd_jwt_type.kbHeader, _lukas_j_han_sd_jwt_type.kbPayload>>;
    keys(endcodedSDJwt: SDJWTCompact): Promise<string[]>;
    presentableKeys(endcodedSDJwt: SDJWTCompact): Promise<string[]>;
    getClaims(endcodedSDJwt: SDJWTCompact): Promise<unknown>;
}

export { Jwt, type JwtData, KBJwt, SDJwt, type SDJwtData, SDJwtInstance, createDecoy, listKeys, pack };
